@@ -4,7 +4,9 @@ import os.path as op
 from pyparty.tools import Canvas, concat_canvas
 from pyparty.tools.grids import Grid, CartesianGrid
 from pyparty.utils import showim, any2rgb, crop, splot, to_normrgb
+from pyparty.multi import MultiCanvas
 
-#  Borrowing this from skimage
+#  Borrowing API from skimage
 pkg_dir = op.abspath(op.dirname(__file__))
-data_dir = op.join(pkg_dir, 'data')
\ No newline at end of file
+data_dir = op.join(pkg_dir, 'data')
+bundled_dir = op.join(pkg_dir, 'bundled')
\ No newline at end of file
