@@ -8,8 +8,7 @@ import matplotlib.pyplot as plt
 from traits.api import HasTraits, List, Instance, Str, Dict, Property
 
 from pyparty.tools import Canvas, ParticleManager
-from pyparty.utils import _get_ccycle, mem_address, \
-    _parse_generator, _parse_ax, rgb2uint, rand_color, multi_axes
+import pyparty.utils as putil
 import pyparty.tools.arraytools as ptools
 from pyparty.config import MAXOUT, PADDING, ALIGN
 
@@ -48,29 +47,30 @@ def _parse_names(names, default_names):
 
 
 def multi_mask(img, *names, **kwargs):
-    """ 
-    """
-    astype = kwargs.pop('astype', tuple)
-    ignore = kwargs.pop('ignore', 0)
-    names = list(names)
+    """ Return a list/tuple/generator of masks of an image, one for each unique
+    label found in a colored img. EVERYTHING DOWN-CONVERTED TO UINT: NECESSARY
+    FOR MORPHOLOGY.LABELS ANYWAY, ALTHOUGH POSSIBLY SHOULD DO RGB2GRAY OR 255
+    LABELS IS MAX"""
     
+    astype = kwargs.pop('astype', tuple)
+    ignore = kwargs.pop('ignore', 'black')
+    names = list(names)    
+    
+    # THIS WILL DRASTICALLY REDUCE NUMBER OF UNIQUE LABELS ALLOWED RIGHT? 
+    # SINCE IT ONLY HAS 255 POSSIBLE VALUES?!
     if img.ndim == 3:
-        img = rgb2uint(img, warnmsg=True)
+        img = putil.rgb2uint(img, warnmsg=True)
     
     # Labels requires on grayscale; npunique also doesn't play nice w/ rgb
     
-    if ignore == 'black':
+    if ignore == 'black' or ignore == (0,0,0):
         ignore = 0
-        if img.ndim == 3:
-            ignore = (0.0,0.0,0.0)
 
-    elif ignore == 'white':
-        ignore = 255    
-        if img.ndim == 3:
-            ignore = (1.0,1.0,1.0)
+    elif ignore == 'white' or ignore == (1,1,1):
+        ignore = 255
     
     unique = ptools.unique(img)
-
+    
     if ignore not in unique:
         logger.warn("Ignore set to %s but was not found on image." % ignore)
     else:
@@ -80,7 +80,8 @@ def multi_mask(img, *names, **kwargs):
             
     # Make the mask dict as generator
     out = ((str(names[idx]), (img==v)) for idx, v in enumerate(unique))
-    return _parse_generator(out, astype)
+    return putil._parse_generator(out, astype)
+
 
 class MultiError(Exception):
     """ """
@@ -133,7 +134,7 @@ class MultiCanvas(HasTraits):
         """ Gives a list of colors based on stored colors AND
         plot color cycle."""
         
-        mplcolors = _get_ccycle(upto=len(self))
+        mplcolors = putil._get_ccycle(upto=len(self))
         colors = []
         
         for idx, name in enumerate(self.names):
@@ -154,7 +155,7 @@ class MultiCanvas(HasTraits):
         """        
         gen_out = ( (self.names[idx], c.pbinary) for idx, c 
                         in enumerate(self.canvii) )
-        return _parse_generator(gen_out, astype)
+        return putil._parse_generator(gen_out, astype)
     
     
     def to_canvas(self, mapnames=False, mapcolors=False):
@@ -235,7 +236,7 @@ class MultiCanvas(HasTraits):
             gen_out = ( (self.names[idx], getattr(c, attr)) for idx, c 
                         in enumerate(self.canvii) )
 
-        return _parse_generator(gen_out, as_type)        
+        return putil._parse_generator(gen_out, as_type)        
         
 
     # ------------------
@@ -279,10 +280,10 @@ class MultiCanvas(HasTraits):
             raise MultiError("showstyle must be show or patchshow, "
                              "not %s" % showstyle)
         
-        axes, kwargs = _parse_ax(*args, **kwargs)	
+        axes, kwargs = putil._parse_ax(*args, **kwargs)	
                 
         if not axes:
-            axes, kwargs = multi_axes(len(self), **kwargs)
+            axes, kwargs = putil.multi_axes(len(self), **kwargs)
 
         if len(axes) < len(self):
             logger.warn("MultiCanvas has %s canvas, but only %s axes recieved"
@@ -348,7 +349,7 @@ class MultiCanvas(HasTraits):
         else:
             autopct = chartkwargs.get('autopct', None)
     
-        axes, chartkwargs = _parse_ax(*chartargs, **chartkwargs)	
+        axes, chartkwargs = putil._parse_ax(*chartargs, **chartkwargs)	
         if not axes:
             fig, axes = plt.subplots()       
         
@@ -436,7 +437,7 @@ class MultiCanvas(HasTraits):
         if 'color' not in histkwargs:
             histkwargs['color'] = self._request_plotcolors()
             
-        axes, histkwargs = _parse_ax(*histargs, **histkwargs)	
+        axes, histkwargs = putil._parse_ax(*histargs, **histkwargs)	
         if not axes:
             fig, axes = plt.subplots()        
         
@@ -470,11 +471,47 @@ class MultiCanvas(HasTraits):
             axes.legend()
         return axes
 
+
+
+    def scatter(self, *args, **kwargs):
+        """ attr1/2 default to canvas.scatter """
+        
+        multi= kwargs.pop('multi', False)
+        colors = kwargs.pop('colors', self._request_plotcolors())
+        annotate = kwargs.pop('annotate', False)
+        
+        axes, kwargs = putil._parse_ax(*args, **kwargs)
+
+        # Multiplot style
+        if multi:
+            raise NotImplementedError
+    
+        # Overlaid plot style
+        else:
+            if not axes:
+                fig, axes = plt.subplots()
+            
+            for idx, c in enumerate(self.canvii):
+                # Only pass annotate once to avoid re-write axis labels/title
+                if idx == 0 and annotate:
+                    addlabel = True
+                else:
+                    addlabel = False
+                c.scatter(axes, color=colors[idx], annotate=addlabel, **kwargs)
+                
+
+            if annotate:
+                axes.legend(self.names)
+        
+        return axes    
+        
+        
     def summary(self):
         """ """
         # Breakdown of c things in names
         NotImplemented
 
+
     def copy(self): #, **kwargs):
         """ Copy multicanvas """
         names, canvii = [], []
@@ -482,6 +519,7 @@ class MultiCanvas(HasTraits):
         canvii[:] = self.canvii[:]
         return MultiCanvas(names=names, canvii=canvii, 
                              _mycolors = self.mycolors)
+
         
     def pop(self, idx):
         self.names.pop(idx)
@@ -499,6 +537,31 @@ class MultiCanvas(HasTraits):
     def insert(self, idx, name, canvas):
         self.names.insert(idx, name)
         self.canvii.insert(idx, canvas)    
+        
+
+    def reorder(self, *names_or_int, **kwds):
+        """ Reorder based on names or integers.  Does not enforce length or
+        non-duplication checks. Use at your own discretion"""
+        
+        inplace = kwds.pop('inplace', False)       
+       
+        def _to_index(item):
+            if isinstance(item, str):
+                return self.names.index(item)
+            return item
+        
+        indices = map(_to_index, names_or_int)
+        names = [self.names[i] for i in indices]
+        canvii = [self.canvii[i] for i in indices]
+        
+        if inplace:
+            self.names = names
+            self.canvii = canvii
+            
+        else:
+            return MultiCanvas(names=names, canvii=canvii,
+                               _mycolors = self.mycolors)
+            
               
     
     def set_colors(self, *colors, **kwcolors):
@@ -518,7 +581,7 @@ class MultiCanvas(HasTraits):
         if kwcolors:
             names, colors = zip(*kwcolors.items())
             if fillnull:
-                colors = [c if c else rand_color(style='hex') for c in colors]                
+                colors = [c if c else putil.rand_color(style='hex') for c in colors]                
             
             for idx, name in enumerate(names):
                 if name not in self.names:
@@ -527,7 +590,7 @@ class MultiCanvas(HasTraits):
 
         else: #if *colors
             if fillnull:
-                colors = [c if c else rand_color(style='hex') for c in colors]
+                colors = [c if c else putil.rand_color(style='hex') for c in colors]
         
             for idx, name in enumerate(self.names):
                 self._mycolors[name] = colors[idx]
@@ -577,7 +640,7 @@ class MultiCanvas(HasTraits):
     @property
     def _address(self):
         """ Property to make easily accesible by multicanvas """
-        return mem_address(super(MultiCanvas, self).__repr__())   
+        return putil.mem_address(super(MultiCanvas, self).__repr__())   
 
 
     # ---------------
@@ -650,10 +713,15 @@ class MultiCanvas(HasTraits):
     def items(self):
         return zip(self.names, self.canvii)
     
+    @property
+    def pcount(self):
+        """ Also used in __repr__ """
+        return sum(len(c) for c in self.canvii)
+    
 
     def __repr__(self):
-        outstring = "%s (%s): " % \
-            (self.__class__.__name__, self._address)     
+        outstring = "%s (%s) %s: " % \
+            (self.__class__.__name__, self._address, self.pcount)     
         Ln = len(self)
         
         if Ln == 0:                
@@ -702,37 +770,185 @@ class MultiCanvas(HasTraits):
         for p in ptypes:
             canvii.append(canvas.of_ptypes(p))
         return cls(names=names, canvii=canvii)          
+    
 
     @classmethod
     def from_labeled(cls, img, *names, **pmankwargs):
         """ Labels an image and creates multi-canvas, one for each species
-        in the image."""
+        in the image.  Since labels require gray image, putil.rgb2uint used 
+        internally.
         
-        ignore = pmankwargs.pop('ignore', 0)        
+        Attributes
+        ----------
+        mapper : iterable of pairs, ordered dict or dict
+            Specifies which colors correspond to which labels.  Order of labels
+            is also used as order of multicanvas names, so if using dict,
+            this will be scrambled.
+            
+        Examples
+        --------
+
+        mc = MultiCanvas.from_labeled(image, singles, doubles)
+        mc = MultiCanvas.from_labeled(image, 
+                 mapper=(('singles','red'), ('doubles',(1,1,0)) ))
+                 
+        """
+        
+        ignore = pmankwargs.pop('ignore', 'black')        
         neighbors = pmankwargs.pop('neighbors', 4)
         maximum = pmankwargs.pop('maximum', MAXDEFAULT)
+        mapper = pmankwargs.pop('mapper', [])
+        storecolors = pmankwargs.pop('storecolors', True)
         
-        name_masks = multi_mask(img, *names, astype=tuple, ignore=ignore)
+        # Retro-support *names positional arg by converting to mapper
+        # since mapper already works downstream
+        if names:
+            if mapper:
+                raise MultiError("Names must be positional arguments or"
+                                 " mapper, but not both.")
+            mapper = zip(names, putil._get_ccycle(upto=len(names)) )
+
+        # BAD PRACTICE TO USE DICT CUZ UNSORTED
+        if isinstance(mapper, dict):
+            mapper = mapper.items()
+        if mapper:
+            cnames, colors = zip(*mapper)
+        else:
+            cnames, colors = [], []
+
+        # TEST ON GRAY IMAGE
+        if img.ndim == 3:
+            img = putil.any2rgb(img)
+            unique = [tuple(v) for v in ptools.unique(img)]   
+            threechan = True
+        else:
+#            img = putil.any2uint(img)
+            unique = [float(v) for v in ptools.unique(img)]   
+            threechan=False
+
+        # Redundant w/ name masks but oh well
+        if ignore is None:
+            color_ignore = []
+            
+        elif ignore == 'black' or ignore == (0,0,0) or ignore == 0:
+            color_ignore = [0, (0,0,0)]
+
+        elif ignore == 'white' or ignore == (1,1,1) or ignore == 255:
+            color_ignore = [255, (1,1,1), (255,255,255)]
+            
+        # A list of values, take besut guess
+        else:
+            color_ignore = zip(*putil.guess_colors(ignore, unique, 
+                                                   threechan=threechan))[1]
+    
+        # Store indicies to ignore 
+        index_ignore = [idx for idx, v in enumerate(unique) if v in color_ignore]
+        unique = [v for v in unique if v not in color_ignore]
+
+        
+        # Sort by gray values; multimask will expect this
+        color_gray = zip(map(putil.any2rgb, unique), map(putil.any2uint, unique) )
+        unique, grayunique = zip(*sorted(color_gray, key=operator.itemgetter(1)))
+                      
+        if img.ndim < 3:
+            unique = grayunique
+            
+        _colormapper = dict(putil.guess_colors(colors, unique, threechan=threechan))
+        mout =[(str(c), c) for c in unique]
+        
+        _tempmap = {}
+        for k,v in mapper:
+            _tempmap[k] = _colormapper[v]
+        _tempmap = dict((str(v),k) for k,v in _tempmap.items())
+        
+        def _tryeval(val):
+            
+            def r2(x): return round(x,2)
+
+            try:
+                val = eval(val)
+            except NameError:
+                return val
+            
+            try:
+                r,g,b = val
+            except TypeError:
+                return str(r2(val))
+            else:
+                return str(tuple(map(r2, val)))
+        
+        mapper = []
+        for k,v in mout:
+            if k in _tempmap:
+                kout = _tempmap[k]
+            else:
+                kout = k
+                
+            kout = _tryeval(kout)
+            mapper.append((kout,v))
+       
+                        
+        name_masks = multi_mask(img, *names, astype=tuple, ignore=None)
         if len(name_masks) > maximum:
             raise MultiError("%s labels found, exceeding maximum of %s"
-                " increasing maximum may result in slowdown" % maximum)
-        
-        canvii = []
-        for (name, mask) in name_masks:
-            labels = morphology.label(mask, neighbors, background=False)                                 
-            particles = ParticleManager.from_labels(labels, 
-                            prefix=name, **pmankwargs)
-            canvii.append(Canvas(particles=particles, rez=mask.shape) )
-            
-        return cls(canvii=canvii, names=names)          
+                " increasing maximum may result in slowdown." %
+                (len(name_masks), maximum) )
+                    
+        canvii = [] ; names = []
+        j=0
+        for idx, (name, mask) in enumerate(name_masks):
+            if idx not in index_ignore:
+                labels = morphology.label(mask, neighbors, background=False)                                 
+                particles = ParticleManager.from_labels(labels, 
+                                         prefix=name, **pmankwargs)
+                canvii.append(Canvas(particles=particles, rez=mask.shape) )
+                names.append(mapper[j][0])
+                j += 1
+                
+        mc = cls(canvii=canvii, names=names)              
+
+        # Map colors
+        if storecolors:
+            mc.set_colors(**dict((k,v) for k,v in mapper if k in names))     
+
+        # Reorder names: user cnames first, then remaining in mapper
+        neworder = list(cnames)
+        for idx, (name, v) in enumerate(mapper):
+            if idx not in index_ignore:
+                if name not in cnames:
+                    neworder.append(name)
+        mc.reorder(*neworder, inplace=True)     
+        return mc
+
     
 if __name__ == '__main__':
-    c1 = Canvas.random_circles(n=100, pcolor='yellow')
-    c2 = Canvas.random_triangles(n=100, pcolor='red')
-    c3 = c1+ c2
-    mc =  MultiCanvas.from_canvas(c3, 'dimer', 'trimer')
-    mc.set_colors('r','g')
+    import matplotlib.pyplot as plt
 
-    mcout = mc[0:2]
-    mc.hist(xlim=(2000,50000), attr='equivalent_diameter', bins=30)
+    from skimage.io import imread
+    from pyparty.utils import crop
+#    img = imread('/home/glue/Desktop/imgproc_supplemental/images/Test_Data/Ilastik_Analysis_Final/class_10_labels/Noise_class_10labels_modified.png')
+    from pyparty.data import nanolabels
+    img = nanolabels()
+#    print ptools.unique(img)
+
+  #  plt.imshow(img, cmap='spectral')
+ #   plt.show()
+
+    img = crop(img, (0,0,512,512))
+    mc = MultiCanvas.from_labeled(img, 'gay', 'fag',
+                                  storecolors=False, 
+                                  ignore=0,
+                                  #mapper=[ 
+                                      #('singles', 1),
+                                      #('dimers', 2),
+                                      #('trimer', 3),
+                                      #('bigs',  4)
+                                      #],
+                                  )
+    
+                                  
+    mc.show(annotate=False, names=True, nolabel=True)
+#   plt.imshow(nanolabels())
     plt.show()
+
+    
\ No newline at end of file
