@@ -125,7 +125,7 @@ def array2sphere(xyz_array):
     
     
 def nearest(array, value):
-    """Find nearest value in an array, return index."""
+    """Find nearest value in an array, RETURN INDEX."""
     return (np.abs(array-value)).argmin()
     
     
@@ -140,8 +140,9 @@ def slice_by_value(array, vi=0, vf=None):
     
 def unique(array):
     """ Find unique values in array of arbitrary ndim.  If array.ndim < 3,
-    returns np.unique.  If 3 or greater, returns values as expected; 
-    whereas np.unique always flattens, and hence fails for rgb images.
+    returns np.unique(). Else, returns values sorted by sum of individual
+    elements. np.unique always flattens, so is not suitable rgb images.
+    
     """
     if array.ndim < 3:
         return np.unique(array)
@@ -154,7 +155,7 @@ def unique(array):
     array_reshaped = array.reshape(L*W, rest)    
     o = [tuple(row) for row in array_reshaped]
     o_unique = tuple(set(o))
-    return np.array(o_unique)
+    return sorted(np.array(o_unique), key=sum)
     
 # Set operations
 def _parse_set(array1, array2):
@@ -162,7 +163,7 @@ def _parse_set(array1, array2):
     s1, s2 = array1.shape, array2.shape
     type1, type2 = array1.dtype, array2.dtype
     if s1 != s2:
-        raise ArraySetError("Shape mismatch: %s vs. %s" % (ndim1, ndim2))
+        raise ArraySetError("Shape mismatch: %s vs. %s" % (s1, s2))
     if type1 != type2:
         raise ArraySetError("Dtype mismatch: %s vs. %s" % (type1, type2))
     
