@@ -315,12 +315,14 @@ class Canvas(HasTraits):
         title = kwargs.pop('title', None)        
         bgonly = kwargs.pop('bgonly', False)
         annotate = kwargs.pop('annotate', False)
+        zoom = kwargs.pop('zoom', None)
 
         grid = kwargs.pop('grid', False)
         gcolor = kwargs.pop('gcolor', None)
         gstyle = kwargs.pop('gstyle', None)
         gunder = kwargs.pop('gunder',False)
         pmap = kwargs.pop('pmap', None)
+        nolabel = kwargs.pop('nolabel', None)
         
         alpha = kwargs.get('alpha', None)
         edgecolor = kwargs.get('edgecolor', None)
@@ -363,6 +365,10 @@ class Canvas(HasTraits):
         else:
             bg = self.background
             
+        if zoom:
+            xi, yi, xf, yf = zoom
+            bg = crop(bg, zoom) 
+                        
         #Overwrite axes image
         if not axes:
             fig, axes = plt.subplots()
@@ -409,6 +415,18 @@ class Canvas(HasTraits):
         
         axes = self._annotate_plot(axes, annotate, title)    
         
+        if zoom:
+            axes.set_xlim(xi, xf)
+            axes.set_ylim(yf, yi)
+        
+        if nolabel:
+            axes.xaxis.set_visible(False)
+            axes.yaxis.set_visible(False)
+            if nolabel == 'x':
+                axes.yaxis.set_visible(True)
+            elif nolabel == 'y':
+                axes.xaxis.set_visible(True)        
+        
         if save:
             path = _parse_path(save)
             plt.savefig(path, dpi=dpi, bbox_inches=bbox_inches)
@@ -463,6 +481,8 @@ class Canvas(HasTraits):
         gcolor = kwargs.pop('gcolor', None)
         gunder = kwargs.pop('gunder', False)
         gstyle = kwargs.pop('gstyle', None) #NOT USED
+        nolabel = kwargs.pop('nolabel', False)
+        zoom = kwargs.pop('zoom', None)
         
         if gstyle:
             raise CanvasPlotError('"gstyle" only valid for patchshow()')
@@ -524,24 +544,78 @@ class Canvas(HasTraits):
         else:
             image = self._draw_particles(bg, force_binary=PBINARY)
             image[gattr] = gcolor
+            
                         
         # GRAY CONVERT
         if 'cmap' in kwargs:
             image = rgb2uint(image)           
             
+            
         # Matplotlib
         if axes:
             axes.imshow(image, **kwargs)
         else:     
-            axes = plt.imshow(image, **kwargs).axes        
+            axes = plt.imshow(image, **kwargs).axes         
             
         axes = self._annotate_plot(axes, annotate, title)            
+        
+        # SHOW DOESNT ACTUALLY CROP ANYTHING WHEN ZOOMING
+        if zoom:
+            xi, yi, xf, yf = zoom
+            axes.set_xlim(xi, xf)
+            axes.set_ylim(yf, yi)
+
+        if nolabel:
+            axes.xaxis.set_visible(False)
+            axes.yaxis.set_visible(False)
+            if nolabel == 'x':
+                axes.yaxis.set_visible(True)
+            elif nolabel == 'y':
+                axes.xaxis.set_visible(True)
+
         if save:
             path = _parse_path(save)
             skimage.io.imsave(path, image)   
         return axes
 
 
+    def scatter(self, *args, **kwargs):
+        """ plt.scatter wrapper.  Requires keywords attr1, attr2 """
+
+        attr1 = kwargs.pop('attr1', None)
+        attr2 = kwargs.pop('attr2', None)
+        
+        if not attr1 or not attr2:
+            raise CanvasPlotError('Scatter attributes must be specified as'
+                ' keywords (IE c.scatter(attr1=area, attr2=eccentricity ...)')
+    
+        annotate = kwargs.pop('annotate', False)
+        title = kwargs.pop('title', None)
+        fancy = kwargs.pop('fancy', False)
+        
+        axes, kwargs = _parse_ax(*args, **kwargs)
+        if fancy:
+            raise NotImplementedError("Fancy kwarg not yet supported.")
+
+        if not axes:
+            fig, axes = plt.subplots()
+            
+        x, y = getattr(self, attr1), getattr(self, attr2)
+
+        axes.scatter(x, y, **kwargs)
+
+        if annotate:
+            if not title:
+                title = '%s - %s' % (attr1.title(), attr2.title())
+            axes.set_xlabel(attr1)
+            axes.set_ylabel(attr2)
+       
+        if title:
+            axes.set_title(title)
+         
+        return axes
+
+
     def _draw_particles(self, image, force_binary=False):
         """ Draws particles over any image (ie background, background+grid.
         force_binary is a hack to allow for drawing binary particles, useful
@@ -1029,5 +1103,9 @@ class ScaledCanvas(Canvas):
     
     
 if __name__ == '__main__':
-    c=Canvas()
-    print 'hi'
\ No newline at end of file
+    import matplotlib.pyplot as plt
+    c=Canvas.random_circles()
+    print c.rez
+#    c.show(zoom=(40,40,200,400), gcolor='orange')
+#    plt.show()
+    print c.grid.as_tiles(key='2d').keys()
\ No newline at end of file
