@@ -74,24 +74,64 @@ def rand_color(style=None):
     else:
         r = lambda: random.random()
         return  ( r(), r(), r() )
+
+
+def color_distance(rgb1, rgb2):
+    """ Compute absolute difference between 3-channels. """
+    r1, g1, b1 = rgb1
+    r2, g2, b2 = rgb2
+    return abs(r2-r1) + abs(g2-g1) + abs(b2-b1)
     
-def guess_colors(guess, values):
+def guess_colors(guesses, colors, threechan=True):
     """ Match closest via squared channel distance between user-guessed
-    colors and true values.  All are mapped to rgb, so guesses like 'red'
-    are valid.
-    """
-    if not hasattr(guess, '__iter__') and not hasattr(values, '__iter__'):
-        guess, values = [guess], [values]
-
-    if len(guess) != len(values):
-        raise UtilsError('Guesses and values must have equal lengths:'
-                         ' (%s vs %s)' %(len(guess),len(values) ) )
-
-    guess = map(to_normrgb, guess)
-    values = map(to_normrgb, values)
+    colors and true colors.  All are mapped to rgb, so guesses like 'red'
+    are valid.  If threechan, different in each channel is summed (with no weight/bias).  Otherwise,
+    difference of gray values is taken.
     
-    NotImplemented
-    #Returns?
+    Returns
+    -------
+    (guess, closest value) pairs eg if you guessed (.5, .5, .5) and closest was
+    (.5, .7, .5), would return [ ( (.5, .7, .5) , (.5, .5, .5) )]
+
+    """
+    # List being modified in place
+    guesses = guesses[:]
+    colors = colors[:]
+    
+    if not hasattr(guesses, '__iter__'): 
+        guesses = [guesses]
+    
+    if not hasattr(colors, '__iter__'):
+        colors = [colors]
+        
+    if len(guesses) > len(colors):
+        raise UtilsError('Guesses length cannot exceed value length:'
+                         ' (%s vs %s)' %(len(guesses), len(colors)) )
+    pairs = []
+    original_names = guesses[:]
+
+    if threechan:
+        guesses = map(any2rgb, guesses)
+        colors = map(any2rgb, colors)
+        distfcn = color_distance
+    
+    else:
+        guesses = map(any2uint, guesses)
+        colors = map(any2uint, colors)
+        def _subtract(x,y): return abs(y - x)
+        distfcn = _subtract
+
+    for idx, guess in enumerate(guesses):
+        # Pairwise all distances for this guess
+        distance = [distfcn(guess, v) for v in colors]
+
+        # One-line to return min index in a list of floats
+        idx_min = min(enumerate(distance), key=operator.itemgetter(1))[0] 
+        
+        pairs.append( (original_names[idx], colors.pop(idx_min)) )
+
+    return pairs
+    #Returns(guess, closest value) pairs
 
 
 def _pix_norm(value, imax=CBITS):
@@ -153,20 +193,21 @@ def to_normrgb(color):
     # If iterable, assume 3-channel RGB
     if hasattr(color, '__iter__'):
         if len(color) != 3:
-            raise ColorError("Multi-channel color must be 3-channel;"
+            if len(color) == 4:
+                color = color[0:3]
+                logger.warn("4-channel RGBA recieved; ignoring A channel")
+            else:
+                raise ColorError("Multi-channel color must be 3-channel;"
                              " recieved %s" % len(color))
         r, g, b = color
         if r <= 1 and g <= 1 and b <= 1:
             return (r, g, b)
 
-        elif r >= 1 and g >= 1 and b >= 1:
+        # Any thing like (0, 255, 30) ... uses 255 as upper limit!
+        else:
             r, g, b = map(_pix_norm, (r, g, b) )        
             return (r, g, b)
 
-        else:
-            raise ColorError("Multi-channel color style ambiguous. (r, g, b)"
-                             " elements must all be < 1 or all > 1 (normalized to %s pixels)" 
-                             % CBITS)
 
     if isinstance(color, str):
         if color == 'random':
@@ -197,6 +238,9 @@ def any2rgb(array, name=''):
     name : str
         Name of array which will be referenced in logger messages"""
 
+    if not isinstance(array, np.ndarray):
+        return to_normrgb(array)
+
     # *****
     # Quick way to convert to float (don't use img_as_float becase we want
     # to enforce that upperlimit of 255 is checked
@@ -210,6 +254,10 @@ def any2rgb(array, name=''):
         array = array / COLORTYPE[1] 
 
     if array.ndim == 3:
+        # If RGBA
+        if array.shape[2] == 4:
+            array = array[..., 0:3]
+            logger.warn("4-channel RGBA recieved; ignoring A channel")            
         return array 
 
     elif array.ndim == 2:
@@ -243,6 +291,38 @@ def coords_in_image(rr_cc, shape):
     return (rr[mask], cc[mask])            
 
 
+def any2uint(color):
+    
+    def _rgb2grizz(rgb):
+        rgb = to_normrgb(rgb)
+        r,g,b = rgb
+        return int(round(255 *(0.2125 * r + 0.7154 * g + 0.0721 * b),0))
+
+    if isinstance(color, np.ndarray):
+        return rgb2uint(color)
+
+    if isinstance(color, bool):
+        if color:
+            return 255
+        return 0    
+    
+
+    elif isinstance(color, float):
+        color = int(round(color, 0))
+    
+    elif isinstance(color, int):
+        pass
+
+    else:
+        color = _rgb2grizz(color)
+        
+    if color < 0 or color > 255:
+        raise ColorError("Color %s exceeded range 0-255" % str(color) )
+    
+    return color
+
+
+
 def rgb2uint(image, warnmsg=False):
     """ Returns color image as 8-bit unsigned (0-255) int.  Unsigned 8bit gray 
     values are safer to plotting; so enforced throughout pyparty."""
@@ -250,7 +330,7 @@ def rgb2uint(image, warnmsg=False):
         # Try this:
         #	print c.grayimage.max(), c.image.max() * 255, img_as_uint(lena()).max()
 
-    # DOES NOT CHECK IMAGE DIMENSIONS; LEAVES THAT TO CALLING OBJECT
+    # DOES NOT CHECK IMAGE DIMENSIONS; LEAVES THAT TO CALLING OBJECT       
     grayimg = img_as_ubyte( skcol.rgb2gray(image) )
     if warnmsg:
         if isinstance(warnmsg, str):
@@ -374,6 +454,7 @@ def _parse_ax(*args, **kwargs):
 def showim(image, *args, **kwargs):
     """ Similar to imshow with a few more keywords"""
 
+    nolabel = kwargs.pop('nolabel', False)
     if not isinstance(image, np.ndarray):
         raise UtilsError("First argument to showim() must be an ndarray/image, "
                          "got %s instead." % type(image))
@@ -385,6 +466,14 @@ def showim(image, *args, **kwargs):
         axes.imshow(image, **kwargs)
     else:      # matplotlib API asymmetry
         axes = plt.imshow(image, **kwargs).axes        
+        
+    if nolabel:
+        axes.xaxis.set_visible(False)
+        axes.yaxis.set_visible(False)
+        if nolabel == 'x':
+            axes.yaxis.set_visible(True)
+        elif nolabel == 'y':
+            axes.xaxis.set_visible(True)        
 
     if title:
         axes.set_title(title)
@@ -721,6 +810,7 @@ def crop(image, coords):
         image = image[yi:yf, xi:xf]   
     return image
 
+
 def zoom(image, coords, *imshowargs, **imshowkwds):
     """
     Plot zoomed-in region of rectangularly cropped image'
@@ -853,6 +943,6 @@ def zoomshow(image, coords, *imshowargs, **imshowkwds):
     return cropped_image, (ax_full, ax_zoomed)
 
 if __name__ == '__main__':
-    warpedbg = np.random.randint(0, 255, size=(500,500) )
-    zoomshow(warpedbg, (40,40,333,333))
-#    grayhist(warpedbg, xlim='auto', title='Gray Histogram',cdf=True);
\ No newline at end of file
+    c = ( (1,1,0), 'red', 'green') 
+    guess = 'pink'
+    print guess_colors(guess, c, threechan=True)
\ No newline at end of file
